@@ -5,14 +5,13 @@ from ast import Add
 import pandas as pd
 # import os, sys
 # sys.path.append(os.path.normpath(os.path.join(os.path.dirname(os.path.abspath(__file__)), '..','..','py')))
-import dates as mconst # !!! Change this to 'import src.umbitlib.dates' for testing and back to 'import dates' for production upload
+import umbitlib.dates as mconst # !!! Change this to 'import src.umbitlib.dates' for testing and back to 'import umbitlib.dates' for production upload
 import math
 import numpy as np
 import datetime
 import calendar
 from typing import Type, Union, List
-
-
+import sqlalchemy as sa
 
 #################
 # FORMATTING FUNCTIONS
@@ -31,6 +30,173 @@ def human_format(num):
     return '{}{}'.format('{:f}'.format(num).rstrip('0').rstrip('.'), ['', 'K', 'M', 'B', 'T'][magnitude])
 
 
+def convert_seconds(seconds: Union[int, float]) -> tuple:
+    """
+    Converts the given number of seconds into days, hours, minutes, and seconds.
+
+    Args:
+        seconds (Union[int, float]): The number of seconds to convert.
+
+    Returns:
+        tuple: A tuple containing the number of days, hours, minutes, and seconds.
+
+    Raises:
+        ValueError: If the provided value for seconds is negative.
+
+    Example:
+        convert_seconds(3661)
+        Output: (0, 1, 1, 1)
+    """
+    if seconds < 0:
+        raise ValueError("Seconds cannot be negative.")
+
+    # Calculate the number of days
+    days = seconds // (24 * 3600)
+    seconds %= 24 * 3600
+
+    # Calculate the number of hours
+    hours = seconds // 3600
+    seconds %= 3600
+
+    # Calculate the number of minutes
+    minutes = seconds // 60
+    seconds %= 60
+
+    return days, hours, minutes, seconds
+
+
+def convert_data_types(df: pd.DataFrame, conversions: List[Union[str, dict]]) -> pd.DataFrame:
+    """
+    Converts specified columns to target data types in a pandas DataFrame.
+
+    Convert to types = ['str', 'int', 'float', 'datetime', 'date']
+
+    Args:
+        df (pandas.DataFrame): The DataFrame to perform type conversions on.
+        conversions (list): A list of conversions to apply. Each conversion can be a string representing
+            the column name or a dictionary specifying the column name and the target data type.
+
+    Returns:
+        pandas.DataFrame: The DataFrame with converted data types.
+
+    Raises:
+        ValueError: If a conversion entry is not a string or a dictionary.
+        ValueError: If a column specified in the conversion does not exist in the DataFrame.
+        ValueError: If an unsupported data type is specified for conversion.
+
+    Example:
+        data = {
+            'col1':['a','b','c'],
+            'col2':['1',np.nan,'3'],
+            'col3':[True, False, True],
+            'col4':['01-12-2023', None,'06-22-2023'],
+            'col5':['1.1', '2.2', pd.NA]
+        }
+
+        df = pd.DataFrame(data)
+
+        conversions = [
+            {'col2': 'int'},
+            {'col3': 'str'},
+            {'col4': 'date'},
+            {'col5': 'float'}
+        ]
+
+        df = pd.DataFrame(data)
+        result = convert_data_types(df, conversions)
+        print(result.dtypes)
+
+    Notes:
+        - The 'datetime' and 'date' conversions use 'errors='coerce' to handle invalid date values, converting them to NaT.
+        - Normalize will set the time to midnight for each date.
+        - Default pandas behavior coerces integer fields with NaN values to floats.
+    """
+    df_copy = df.copy()
+    try:
+        for conversion in conversions:
+            if isinstance(conversion, str):
+                column = conversion
+                if column not in df_copy.columns:
+                    raise ValueError(f"Column '{column}' does not exist in the DataFrame.")
+                df_copy[column] = df_copy[column].astype(str)
+
+            elif isinstance(conversion, dict):
+                column, target_type = next(iter(conversion.items()))
+                if column not in df_copy.columns:
+                    raise ValueError(f"Column '{column}' does not exist in the DataFrame.")
+                
+                if target_type == 'datetime':
+                    df_copy[column] = pd.to_datetime(df_copy[column], errors='coerce')
+                elif target_type == 'date':
+                    df_copy[column] = pd.to_datetime(df_copy[column], errors='coerce').dt.normalize()
+                elif target_type == 'int':
+                    df_copy[column] = pd.to_numeric(df_copy[column], errors='coerce').astype('Int64')
+                elif target_type == 'float':
+                    df_copy[column] = pd.to_numeric(df_copy[column], errors='coerce')
+                elif target_type == 'str':
+                    df_copy[column] = df_copy[column].astype(str)
+                else:
+                    raise ValueError(f"Unsupported data type '{target_type}' specified for conversion.")
+
+            else:
+                raise ValueError("Each conversion entry should be a string or a dictionary.")
+
+        return df_copy
+
+    except Exception as e:
+        raise ValueError(f"Error occurred during data type conversion: {str(e)}")
+    
+
+def generate_sqlalchemy_dtypes(dataframe):
+    """
+    Generate a dictionary of SQLAlchemy column types based on the data types of a DataFrame's columns.
+
+    This function takes a pandas DataFrame as input and analyzes its column data types to create a dictionary
+    that maps column names to their corresponding SQLAlchemy column types. The generated types can be useful
+    for defining a database table schema using SQLAlchemy.
+
+    Parameters:
+        dataframe (pandas.DataFrame): The input DataFrame containing the data and column types.
+
+    Returns:
+        dict: A dictionary where keys are column names and values are SQLAlchemy column types.
+
+    Example:
+        import pandas as pd
+        from sqlalchemy import types as sa
+
+        data = {'Name': ['Alice', 'Bob', 'Charlie'],
+                'Age': [25, 30, 28],
+                'Height': [160.5, 175.2, 162.0],
+                'IsStudent': [True, False, False]}
+        df = pd.DataFrame(data)
+        dtypes_dict = generate_sqlalchemy_dtypes(df)
+        # Resulting dictionary:
+        # {'Name': sa.types.VARCHAR(length=7),
+        #  'Age': sa.types.INTEGER(),
+        #  'Height': sa.types.FLOAT(),
+        #  'IsStudent': sa.types.BOOLEAN()}
+    """
+    sqlalchemy_dtypes = {}
+    
+    for col_name, col_dtype in dataframe.dtypes.items():
+        if pd.api.types.is_integer_dtype(col_dtype):
+            sqlalchemy_dtypes[col_name] = sa.types.INTEGER()
+        elif pd.api.types.is_float_dtype(col_dtype):
+            sqlalchemy_dtypes[col_name] = sa.types.FLOAT()
+        elif pd.api.types.is_bool_dtype(col_dtype):
+            sqlalchemy_dtypes[col_name] = sa.types.BOOLEAN()
+        elif pd.api.types.is_string_dtype(col_dtype):
+            max_length = dataframe[col_name].astype(str).apply(len).max()
+            sqlalchemy_dtypes[col_name] = sa.types.VARCHAR(length=max_length)
+        elif pd.api.types.is_datetime64_any_dtype(col_dtype):
+            sqlalchemy_dtypes[col_name] = sa.types.TIMESTAMP()
+        else:
+            max_length = dataframe[col_name].astype(str).apply(len).max()
+            sqlalchemy_dtypes[col_name] = sa.types.VARCHAR(length=max_length)
+    
+    return sqlalchemy_dtypes
+    
     
 #################
 # ARITHMETIC FUNCTIONS
