@@ -4,8 +4,8 @@ import time
 import cx_Oracle
 import os
 import os.path
-from constants import *  # !!! Change this to 'import src.umbitlib.constants' for testing and back to 'import constants' for production upload
-from dates import TRGTMON_YYYYMMDD  # !!! Change this to 'import src.umbitlib.dates' for testing and back to 'import dates' for production upload
+from umbitlib.constants import * # !!! Change this to 'import src.umbitlib.constants' for testing and back to 'from umbitlib.constants import *' for production upload
+from umbitlib.dates import * # !!! Change this to 'import src.umbitlib.dates' for testing and back to 'from umbitlib.dates import TRGTMON_YYYYMMDD' for production upload
 import warnings
 
 
@@ -24,9 +24,10 @@ def excel_credentials():
     Example:        
         username, password = excel_credentials()
     """    
+    
+    username = os.getlogin()
     try:  
-        data_source_file_oracle = "\\\\hsc-evs03.ad.utah.edu\\users\\" + \
-            os.getlogin() + "\\Data Sources\\UIDPWD.xlsx"
+        data_source_file_oracle = f"//ad.utah.edu/uuhc/users/{username}/Data Sources/UIDPWD.xlsx"
         wb_credentials_oracle = load_workbook(data_source_file_oracle)
         sheet_oracle = wb_credentials_oracle['Sheet1']
         username = sheet_oracle['A2'].value
